t_notebook = "notebook"
t_asset = "asset"
t_dependency = "dependency"
t_scheduler = "scheduler"
t_secret = "secret"

t_job = "job"
t_env = "env"
t_log = "log"

t_list_output = "list_output"
t_list_histo = "list_histo"
t_output = "output"
t_production = "prod"
t_histo = "history"
t_list = "list"
t_send = "send"
t_add = "installed"
t_delete = "delete"
t_update = "edited"
t_start = "started"
t_busy = "busy"
t_skip = "skiped"
t_error = "error"
t_health = "healthy"
t_main = "main"
